@@ -704,6 +704,9 @@ static void get_managed_objects_callback(struct l_dbus_message *message,
 	while (l_dbus_message_iter_next_entry(&objects, &path, &object))
 		proxy_interface_create(path, &object);
 
+	if (command_needs_no_agent())
+		goto no_agent;
+
 	if (!agent_manager_register_agent()) {
 		display_error("Failed to register Agent.\n");
 
@@ -715,6 +718,7 @@ static void get_managed_objects_callback(struct l_dbus_message *message,
 		return;
 	}
 
+no_agent:
 	if (!command_is_interactive_mode()) {
 		command_noninteractive_trigger();
 
@@ -829,7 +833,8 @@ bool dbus_proxy_exit(void)
 {
 	struct interface_type_desc *desc;
 
-	agent_manager_unregister_agent();
+	if (!command_needs_no_agent())
+		agent_manager_unregister_agent();
 
 	for (desc = __start___interface; desc < __stop___interface; desc++) {
 		if (!desc->exit)
