@@ -28,8 +28,9 @@
 #include <errno.h>
 #include <ell/ell.h>
 
-#include "eap.h"
-#include "eap-private.h"
+#include "src/missing.h"
+#include "src/eap.h"
+#include "src/eap-private.h"
 
 struct eap_md5_state {
 	char *secret;
@@ -41,6 +42,9 @@ static void eap_md5_free(struct eap_state *eap)
 
 	eap_set_data(eap, NULL);
 
+	if (md5->secret)
+		explicit_bzero(md5->secret, strlen(md5->secret));
+
 	l_free(md5->secret);
 	l_free(md5);
 }
@@ -126,6 +130,7 @@ static int eap_md5_check_settings(struct l_settings *settings,
 		password = l_settings_get_string(settings, "Security",
 							password_key_old);
 		if (password) {
+			explicit_bzero(password, strlen(password));
 			l_warn("Setting '%s' is deprecated, use '%s' instead",
 					password_key_old, password_key);
 			return 0;
@@ -139,7 +144,8 @@ static int eap_md5_check_settings(struct l_settings *settings,
 		eap_append_secret(out_missing, EAP_SECRET_REMOTE_PASSWORD,
 					password_key, NULL, identity,
 					EAP_CACHE_TEMPORARY);
-	}
+	} else
+		explicit_bzero(password, strlen(password));
 
 	return 0;
 }
@@ -160,11 +166,8 @@ static bool eap_md5_load_settings(struct eap_state *eap,
 									prefix);
 		secret = l_settings_get_string(settings, "Security",
 								password_key);
-
-		if (!secret) {
-			l_error("Property '%sPassword' is missing.", prefix);
+		if (!secret)
 			return false;
-		}
 	}
 
 	md5 = l_new(struct eap_md5_state, 1);
