@@ -24,6 +24,7 @@
 #include <config.h>
 #endif
 
+#include <errno.h>
 #include <linux/if_ether.h>
 #include <ell/ell.h>
 
@@ -31,6 +32,145 @@
 
 #include "src/nl80211util.h"
 
+typedef bool (*attr_handler)(const void *data, uint16_t len, void *o);
+
+static bool extract_ifindex(const void *data, uint16_t len, void *o)
+{
+	uint32_t *out = o;
+
+	if (len != 4)
+		return false;
+
+	/* ifindex cannot be 0 */
+	if (l_get_u32(data) == 0)
+		return false;
+
+	*out = l_get_u32(data);
+	return true;
+}
+
+static bool extract_name(const void *data, uint16_t len, void *o)
+{
+	const char **out = o;
+
+	if (len < 1)
+		return false;
+
+	if (!memchr(data + 1, 0, len - 1))
+		return false;
+
+	*out = data;
+	return true;
+}
+
+static bool extract_uint64(const void *data, uint16_t len, void *o)
+{
+	uint64_t *out = o;
+
+	if (len != 8)
+		return false;
+
+	*out = l_get_u64(data);
+	return true;
+}
+
+static bool extract_uint32(const void *data, uint16_t len, void *o)
+{
+	uint32_t *out = o;
+
+	if (len != 4)
+		return false;
+
+	*out = l_get_u32(data);
+	return true;
+}
+
+static attr_handler handler_for_type(enum nl80211_attrs type)
+{
+	switch (type) {
+	case NL80211_ATTR_IFINDEX:
+		return extract_ifindex;
+	case NL80211_ATTR_WIPHY:
+	case NL80211_ATTR_IFTYPE:
+		return extract_uint32;
+	case NL80211_ATTR_WDEV:
+	case NL80211_ATTR_COOKIE:
+		return extract_uint64;
+	case NL80211_ATTR_IFNAME:
+	case NL80211_ATTR_WIPHY_NAME:
+		return extract_name;
+	default:
+		break;
+	}
+
+	return NULL;
+}
+
+struct attr_entry {
+	uint16_t type;
+	void *data;
+	attr_handler handler;
+};
+
+int nl80211_parse_attrs(struct l_genl_msg *msg, int tag, ...)
+{
+	struct l_genl_attr attr;
+	va_list args;
+	struct l_queue *entries;
+	const struct l_queue_entry *e;
+	struct attr_entry *entry;
+	uint16_t type;
+	uint16_t len;
+	const void *data;
+	int ret;
+
+	if (!l_genl_attr_init(&attr, msg))
+		return -EINVAL;
+
+	va_start(args, tag);
+	entries = l_queue_new();
+	ret = -ENOSYS;
+
+	while (tag != NL80211_ATTR_UNSPEC) {
+		entry = l_new(struct attr_entry, 1);
+
+		entry->type = tag;
+		entry->data = va_arg(args, void *);
+		entry->handler = handler_for_type(tag);
+		l_queue_push_tail(entries, entry);
+
+		if (!entry->handler)
+			goto done;
+
+		tag = va_arg(args, enum nl80211_attrs);
+	}
+
+	va_end(args);
+
+	while (l_genl_attr_next(&attr, &type, &len, &data)) {
+		for (e = l_queue_get_entries(entries); e; e = e->next) {
+			entry = e->data;
+
+			if (entry->type == type)
+				break;
+		}
+
+		if (!e)
+			continue;
+
+		if (!entry->handler(data, len, entry->data)) {
+			ret = -EINVAL;
+			goto done;
+		}
+	}
+
+	ret = 0;
+
+done:
+	l_queue_destroy(entries, l_free);
+	return ret;
+}
+
 struct l_genl_msg *nl80211_build_new_key_group(uint32_t ifindex, uint32_t cipher,
 					uint8_t key_id, const uint8_t *key,
 					size_t key_len, const uint8_t *ctr,
@@ -194,3 +334,35 @@ const void *nl80211_parse_get_key_seq(struct l_genl_msg *msg)
 
 	return data;
 }
+
+struct l_genl_msg *nl80211_build_cmd_frame(uint32_t ifindex,
+						const uint8_t *addr,
+						const uint8_t *to,
+						uint32_t freq,
+						struct iovec *iov,
+						size_t iov_len)
+{
+	struct l_genl_msg *msg;
+	struct iovec iovs[iov_len + 1];
+	const uint16_t frame_type = 0x00d0;
+	uint8_t action_frame[24];
+
+	memset(action_frame, 0, 24);
+
+	l_put_le16(frame_type, action_frame + 0);
+	memcpy(action_frame + 4, to, 6);
+	memcpy(action_frame + 10, addr, 6);
+	memcpy(action_frame + 16, to, 6);
+
+	iovs[0].iov_base = action_frame;
+	iovs[0].iov_len = sizeof(action_frame);
+	memcpy(iovs + 1, iov, sizeof(*iov) * iov_len);
+
+	msg = l_genl_msg_new_sized(NL80211_CMD_FRAME, 128 + 512);
+
+	l_genl_msg_append_attr(msg, NL80211_ATTR_IFINDEX, 4, &ifindex);
+	l_genl_msg_append_attr(msg, NL80211_ATTR_WIPHY_FREQ, 4, &freq);
+	l_genl_msg_append_attrv(msg, NL80211_ATTR_FRAME, iovs, iov_len + 1);
+
+	return msg;
+}
