@@ -43,15 +43,13 @@ enum ie_rsn_akm_suite wiphy_select_akm(struct wiphy *wiphy,
 					struct scan_bss *bss,
 					bool fils_capable_hint);
 
-bool wiphy_parse_id_and_name(struct l_genl_attr *attr, uint32_t *out_id,
-				const char **out_name);
-
 struct wiphy *wiphy_find(int wiphy_id);
 
 struct wiphy *wiphy_create(uint32_t wiphy_id, const char *name);
 void wiphy_create_complete(struct wiphy *wiphy);
 bool wiphy_destroy(struct wiphy *wiphy);
-void wiphy_update_from_genl(struct wiphy *wiphy, struct l_genl_msg *msg);
+void wiphy_update_from_genl(struct wiphy *wiphy, const char *name,
+				struct l_genl_msg *msg);
 
 bool wiphy_constrain_freq_set(const struct wiphy *wiphy,
 						struct scan_freq_set *set);
@@ -69,6 +67,7 @@ uint8_t wiphy_get_max_num_ssids_per_scan(struct wiphy *wiphy);
 bool wiphy_supports_iftype(struct wiphy *wiphy, uint32_t iftype);
 bool wiphy_supports_adhoc_rsn(struct wiphy *wiphy);
 bool wiphy_can_offchannel_tx(struct wiphy *wiphy);
+bool wiphy_supports_qos_set_map(struct wiphy *wiphy);
 const char *wiphy_get_driver(struct wiphy *wiphy);
 const char *wiphy_get_name(struct wiphy *wiphy);
 const uint8_t *wiphy_get_permanent_address(struct wiphy *wiphy);
